import { Stack } from 'stack-typescript';
import { UndoableCommand } from './commands/UndoableCommand.js';
import { CloseMergeRequestCommand } from './commands/CloseMergeRequestCommand.js';
import { CommitCommand } from './commands/CommitCommand.js';
import { CreateBranchCommand } from './commands/CreateBranchCommand.js';
import { CreateFileCommand } from './commands/CreateFileCommand.js';
import { CreateGroupCommand } from './commands/CreateGroupCommand.js';
import { CreateMergeRequestCommand } from './commands/CreateMergeRequestCommand.js';
import { CreateOrRenameMergeRequestCommand } from './commands/CreateOrRenameMergeRequestCommand.js';
import { CreateProjectCommand } from './commands/CreateProjectCommand.js';
import { CreateTagCommand } from './commands/CreateTagCommand.js';
import { DeleteBranchCommand } from './commands/DeleteBranchCommand.js';
import { DeleteGroupCommand } from './commands/DeleteGroupCommand.js';
import { DeleteMergeRequestCommand } from './commands/DeleteMergeRequestCommand.js';
import { DeleteProjectCommand } from './commands/DeleteProjectCommand.js';
import { DeleteTagCommand } from './commands/DeleteTagCommand.js';
import { DownloadRepositoryCommand } from './commands/DownloadRepositoryCommand.js';
import { GetFileCommand } from './commands/GetFileCommand.js';
import { GetGroupCommand } from './commands/GetGroupCommand.js';
import { GetMergeRequestCommand } from './commands/GetMergeRequestCommand.js';
import { GetMergeRequestsByBranchesCommand } from './commands/GetMergeRequestsByBranchesCommand.js';
import { GetProjectCommand } from './commands/GetProjectCommand.js';
import { GetProjectMembersCommand } from './commands/GetProjectMembersCommand.js';
import { GetProjectsCommand } from './commands/GetProjectsCommand.js';
import { GetUserDataCommand } from './commands/GetUserDataCommand.js';
import { MergeMergeRequestCommand } from './commands/MergeMergeRequestCommand.js';
import { RefreshAccessTokenCommand } from './commands/RefreshAccessTokenCommand.js';
import { RenameGroupCommand } from './commands/RenameGroupCommand.js';
import { RenameMergeRequestCommand } from './commands/RenameMergeRequestCommand.js';
import { RenameProjectCommand } from './commands/RenameProjectCommand.js';
import { ReopenMergeRequestCommand } from './commands/ReopenMergeRequestCommand.js';
import { RequestAccessTokenCommand } from './commands/RequestAccessTokenCommand.js';
import { RevertCommitCommand } from './commands/RevertCommitCommand.js';
import { SaveFileCommand } from './commands/SaveFileCommand.js';
import { SetDefaultBranchCommand } from './commands/SetDefaultBranchCommand.js';
import { CheckAndMergeCommand } from './commands/CheckAndMergeCommand.js';
import { GetGroupMembersCommand } from './commands/GetGroupMembersCommand.js';
export class GitLabSession {
    constructor(url) {
        this.commandStack = new Stack();
        this.url = url;
    }
    async executeCommand(cmd) {
        const execution = await cmd.execute();
        if (cmd instanceof UndoableCommand) {
            this.commandStack.push(cmd);
        }
        return execution;
    }
    async undo() {
        if (this.commandStack.size > 0) {
            const cmd = this.commandStack.pop();
            return await cmd.undo();
        }
        else {
            return;
        }
    }
    async rollback() {
        for (let i = 0; i < this.commandStack.size; i++) {
            const cmd = this.commandStack.pop();
            await cmd.undo();
        }
    }
    async checkAndMerge(token, id, mergeRequestIid, shouldRemoveSourceBranch) {
        return await this.executeCommand(new CheckAndMergeCommand(this.url, token, id, mergeRequestIid, shouldRemoveSourceBranch));
    }
    async closeMergeRequest(token, id, mergeRequestIid) {
        return await this.executeCommand(new CloseMergeRequestCommand(this.url, token, id, mergeRequestIid));
    }
    async commit(token, projectId, actions, message, branch) {
        return await this.executeCommand(new CommitCommand(this.url, token, projectId, actions, message, branch));
    }
    async createBranch(token, projectId, branchName, ref) {
        return await this.executeCommand(new CreateBranchCommand(this.url, token, projectId, branchName, ref));
    }
    async createFile(token, id, filePath, branch, content) {
        return await this.executeCommand(new CreateFileCommand(this.url, token, id, filePath, branch, content));
    }
    async createGroup(token, name, namespaceId) {
        return await this.executeCommand(new CreateGroupCommand(this.url, token, name, namespaceId));
    }
    async createMergeRequest(token, id, title, sourceBranch, targetBranch) {
        return await this.executeCommand(new CreateMergeRequestCommand(this.url, token, id, title, sourceBranch, targetBranch));
    }
    async createOrRenameMergeRequest(token, id, title, sourceBranch, targetBranch) {
        return await this.executeCommand(new CreateOrRenameMergeRequestCommand(this.url, token, id, title, sourceBranch, targetBranch));
    }
    async createProject(token, name, namespaceId, topics, branch) {
        return await this.executeCommand(new CreateProjectCommand(this.url, token, name, namespaceId, topics, branch));
    }
    async createTag(token, projectId, tag, ref) {
        return await this.executeCommand(new CreateTagCommand(this.url, token, projectId, tag, ref));
    }
    async deleteBranch(token, projectId, branchName) {
        return await this.executeCommand(new DeleteBranchCommand(this.url, token, projectId, branchName));
    }
    async deleteGroup(token, id) {
        return await this.executeCommand(new DeleteGroupCommand(this.url, token, id));
    }
    async deleteMergeRequest(token, id, mergeRequestIid) {
        return await this.executeCommand(new DeleteMergeRequestCommand(this.url, token, id, mergeRequestIid));
    }
    async deleteProject(token, id) {
        return await this.executeCommand(new DeleteProjectCommand(this.url, token, id));
    }
    async deleteTag(token, projectId, tag) {
        return await this.executeCommand(new DeleteTagCommand(this.url, token, projectId, tag));
    }
    async downloadRepository(token, id, ref) {
        return await this.executeCommand(new DownloadRepositoryCommand(this.url, token, id, ref));
    }
    async getFile(token, id, filePath, branch) {
        return await this.executeCommand(new GetFileCommand(this.url, token, id, filePath, branch));
    }
    async getGroup(token, id) {
        return await this.executeCommand(new GetGroupCommand(this.url, token, id));
    }
    async getGroupMembers(token, id, perPage = 100) {
        return await this.executeCommand(new GetGroupMembersCommand(this.url, token, id, perPage));
    }
    async getMergeRequest(token, id, mergeRequestIid) {
        return await this.executeCommand(new GetMergeRequestCommand(this.url, token, id, mergeRequestIid));
    }
    async getMergeRequestsByBranches(token, id, sourceBranch, targetBranch, state) {
        return await this.executeCommand(new GetMergeRequestsByBranchesCommand(this.url, token, id, sourceBranch, targetBranch, state));
    }
    async getProject(token, id) {
        return await this.executeCommand(new GetProjectCommand(this.url, token, id));
    }
    async getProjectMembers(token, id, perPage = 100) {
        return await this.executeCommand(new GetProjectMembersCommand(this.url, token, id, perPage));
    }
    async getProjects(token, topics) {
        return await this.executeCommand(new GetProjectsCommand(this.url, token, topics));
    }
    async getUserData(token) {
        return await this.executeCommand(new GetUserDataCommand(this.url, token));
    }
    async mergeMergeRequest(token, id, mergeRequestIid, shouldRemoveSourceBranch, json) {
        return await this.executeCommand(new MergeMergeRequestCommand(this.url, token, id, mergeRequestIid, shouldRemoveSourceBranch, json));
    }
    async refreshAccessToken(refreshToken, clientId, clientSecret, redirectUri) {
        return await this.executeCommand(new RefreshAccessTokenCommand(this.url, refreshToken, clientId, clientSecret, redirectUri));
    }
    async renameGroup(token, id, name) {
        return await this.executeCommand(new RenameGroupCommand(this.url, token, id, name));
    }
    async renameMergeRequest(token, id, mergeRequestIid, title) {
        return await this.executeCommand(new RenameMergeRequestCommand(this.url, token, id, mergeRequestIid, title));
    }
    async renameProject(token, id, name) {
        return await this.executeCommand(new RenameProjectCommand(this.url, token, id, name));
    }
    async reopenMergeRequest(token, id, mergeRequestIid) {
        return await this.executeCommand(new ReopenMergeRequestCommand(this.url, token, id, mergeRequestIid));
    }
    async requestAccessToken(code, clientId, clientSecret, redirectUri) {
        return await this.executeCommand(new RequestAccessTokenCommand(this.url, code, clientId, clientSecret, redirectUri));
    }
    async revertCommit(token, projectId, sha, branch) {
        return await this.executeCommand(new RevertCommitCommand(this.url, token, projectId, sha, branch));
    }
    async saveFile(token, id, filePath, branch, content) {
        return await this.executeCommand(new SaveFileCommand(this.url, token, id, filePath, branch, content));
    }
    async setDefaultBranch(token, id, branch) {
        return await this.executeCommand(new SetDefaultBranchCommand(this.url, token, id, branch));
    }
}
//# sourceMappingURL=GitLabSession.js.map